@@ -24,29 +24,16 @@
 #include "config.h"
 #endif
 
-#include <string.h>
-#ifndef HAVE_GTK_ONLY
-#include <gnome.h>
-#else
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenuitem.h>
-#endif /* !HAVE_GTK_ONLY */
+#include <glib.h>
+#include <glib/gi18n.h>
 
-#ifdef HAVE_GTK_ONLY
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(GETTEXT_PACKAGE,String)
-#ifdef gettext_noop
-#define N_(String) gettext_noop(String)
-#else
-#define N_(String) (String)
-#endif /* gettext_noop */
-#else
-#define _(String) (String)
-#define N_(String) (String)
-#endif /* ENABLE_NLS */
-#endif /* HAVE_GTK_ONLY */
+#include <string.h>
+
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkcombobox.h>
+#include <gtk/gtkliststore.h>
+#include <gtk/gtkcelllayout.h>
+#include <gtk/gtkcellrenderertext.h>
 
 #include "bacon-cd-selection.h"
 #include "cd-drive.h"
@@ -61,25 +48,21 @@ enum {
 enum {
 	PROP_0,
 	PROP_DEVICE,
+	PROP_FILE_IMAGE,
 };
 
 struct BaconCdSelectionPrivate {
-	gboolean is_entry;
-	GtkWidget *widget;
 	GList *cdroms;
+	gboolean have_file_image;
 };
 
-
-static void bacon_cd_selection_class_init (BaconCdSelectionClass *klass);
-static void bacon_cd_selection_instance_init (BaconCdSelection *bcs);
+static void bacon_cd_selection_init (BaconCdSelection *bcs);
 
 static void bacon_cd_selection_set_property (GObject *object, guint property_id,
 		const GValue *value, GParamSpec *pspec);
 static void bacon_cd_selection_get_property (GObject *object, guint property_id,
 		GValue *value, GParamSpec *pspec);
 
-static void bacon_cd_selection_realize (GtkWidget *widget);
-static void bacon_cd_selection_unrealize (GtkWidget *widget);
 static void bacon_cd_selection_finalize (GObject *object);
 
 static GtkWidgetClass *parent_class = NULL;
@@ -98,34 +81,7 @@ get_drive (BaconCdSelection *bcs, int nr)
 		return item->data;
 }
 
-
-GtkType
-bacon_cd_selection_get_type (void)
-{
-	static GtkType bacon_cd_selection_type = 0;
-
-	if (!bacon_cd_selection_type) {
-		static const GTypeInfo bacon_cd_selection_info = {
-			sizeof (BaconCdSelectionClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) bacon_cd_selection_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL /* class_data */,
-			sizeof (BaconCdSelection),
-			0 /* n_preallocs */,
-			(GInstanceInitFunc) bacon_cd_selection_instance_init,
-			NULL
-		};
-
-		bacon_cd_selection_type = g_type_register_static
-			(GTK_TYPE_VBOX,
-			 "BaconCdSelection", &bacon_cd_selection_info,
-			 (GTypeFlags)0);
-	}
-
-	return bacon_cd_selection_type;
-}
+G_DEFINE_TYPE(BaconCdSelection, bacon_cd_selection, GTK_TYPE_COMBO_BOX)
 
 static void
 bacon_cd_selection_class_init (BaconCdSelectionClass *klass)
@@ -136,11 +92,7 @@ bacon_cd_selection_class_init (BaconCdSelectionClass *klass)
 	object_class = (GObjectClass *) klass;
 	widget_class = (GtkWidgetClass *) klass;
 
-	parent_class = gtk_type_class (gtk_vbox_get_type ());
-
-	/* GtkWidget */
-	widget_class->realize = bacon_cd_selection_realize;
-	widget_class->unrealize = bacon_cd_selection_unrealize;
+	parent_class = gtk_type_class (gtk_combo_box_get_type ());
 
 	/* GObject */
 	object_class->set_property = bacon_cd_selection_set_property;
@@ -151,6 +103,9 @@ bacon_cd_selection_class_init (BaconCdSelectionClass *klass)
 	g_object_class_install_property (object_class, PROP_DEVICE,
 			g_param_spec_string ("device", NULL, NULL,
 				FALSE, G_PARAM_READWRITE));
+	g_object_class_install_property (object_class, PROP_FILE_IMAGE,
+			g_param_spec_boolean ("file-image", NULL, NULL,
+				FALSE, G_PARAM_READWRITE));
 
 	/* Signals */
 	bcs_table_signals[DEVICE_CHANGED] =
@@ -165,171 +120,145 @@ bacon_cd_selection_class_init (BaconCdSelectionClass *klass)
 }
 
 static void
-bacon_cd_selection_instance_init (BaconCdSelection *bcs)
+bacon_cd_selection_init (BaconCdSelection *bcs)
 {
 	bcs->priv = g_new0 (BaconCdSelectionPrivate, 1);
-
-#if defined (__linux__) || defined (__FreeBSD__)
-	bcs->priv->is_entry = FALSE;
-#else
-	bcs->priv->is_entry = TRUE;
-#endif
-
-	bcs->priv->cdroms = NULL;
-}
-
-static void
-bacon_cd_selection_realize (GtkWidget *widget)
-{
-	if (GTK_WIDGET_CLASS (parent_class)->realize != NULL) {
-		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
-	}
-}
-
-static void
-bacon_cd_selection_unrealize (GtkWidget *widget)
-{
-	if (GTK_WIDGET_CLASS (parent_class)->unrealize != NULL) {
-		(* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
-	}
 }
 
 static void
 bacon_cd_selection_finalize (GObject *object)
 {
 	GList *l;
-
 	BaconCdSelection *bcs = (BaconCdSelection *) object;
-	G_OBJECT_CLASS (parent_class)->finalize (object);
 
-	gtk_widget_destroy (bcs->priv->widget);
+	g_return_if_fail (bcs != NULL);
+	g_return_if_fail (BACON_IS_CD_SELECTION (bcs));
 
 	l = bcs->priv->cdroms;
 	while (l != NULL)
 	{
 		CDDrive *cdrom = l->data;
 
-		cd_drive_free (cdrom);
 		l = g_list_remove (l, cdrom);
-		g_free (cdrom);
+		cd_drive_free (cdrom);
 	}
 
+	g_free (bcs->priv);
 	bcs->priv = NULL;
-	bcs = NULL;
+
+	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
+		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	}
 }
 
 static void
-option_menu_device_changed (GtkOptionMenu *option_menu, gpointer user_data)
+combo_device_changed (GtkComboBox *combo, gpointer user_data)
 {
 	BaconCdSelection *bcs = (BaconCdSelection *) user_data;
 	CDDrive *drive;
 	int i;
 
-	i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+	i = gtk_combo_box_get_active (combo);
 	drive = get_drive (bcs, i);
 
 	g_signal_emit (G_OBJECT (bcs),
-			bcs_table_signals[DEVICE_CHANGED],
-			0, drive->device);
+		       bcs_table_signals[DEVICE_CHANGED],
+		       0, drive->device);
 }
 
-static GtkWidget *
-cdrom_option_menu (BaconCdSelection *bcs)
+static void
+cdrom_combo_box (BaconCdSelection *bcs)
 {
 	GList *l;
-	GtkWidget *option_menu, *menu, *item;
 	CDDrive *cdrom;
 
 	bcs->priv->cdroms = scan_for_cdroms (FALSE, FALSE);
 
-	menu = gtk_menu_new();
-	gtk_widget_show(menu);
-
-	option_menu = gtk_option_menu_new ();
-
 	for (l = bcs->priv->cdroms; l != NULL; l = l->next)
 	{
 		cdrom = l->data;
 
-		if (cdrom->display_name == NULL)
+		if (cdrom->display_name == NULL) {
 			g_warning ("cdrom->display_name != NULL failed");
-		item = gtk_menu_item_new_with_label (cdrom->display_name
+		}
+
+		gtk_combo_box_append_text (GTK_COMBO_BOX (bcs),
+				cdrom->display_name
 				? cdrom->display_name : _("Unnamed CDROM"));
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (bcs), 0);
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-
-	if (bcs->priv->cdroms == NULL)
-		gtk_widget_set_sensitive (option_menu, FALSE);
-
-	return option_menu;
+	if (bcs->priv->cdroms == NULL) {
+		gtk_widget_set_sensitive (GTK_WIDGET (bcs), FALSE);
+	}
 }
 
-#ifndef HAVE_GTK_ONLY
-static void
-on_combo_entry_changed (GnomeFileEntry *entry, gpointer user_data)
-{
-	BaconCdSelection *bcs = (BaconCdSelection *) user_data;
-	const char *str;
-	GtkWidget *widget;
-
-	widget = gnome_file_entry_gtk_entry (entry);
-	str = gtk_entry_get_text (GTK_ENTRY (widget));
-
-	g_signal_emit (G_OBJECT (bcs),
-			bcs_table_signals[DEVICE_CHANGED],
-			0, str);
-}
-#endif /* !HAVE_GTK_ONLY */
-
 GtkWidget *
 bacon_cd_selection_new (void)
 {
 	GtkWidget *widget;
 	BaconCdSelection *bcs;
+	GtkCellRenderer *cell;
+	GtkListStore *store;
 
 	widget = GTK_WIDGET
 		(g_object_new (bacon_cd_selection_get_type (), NULL));
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (widget),
+			GTK_TREE_MODEL (store));
+
+	cell = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), cell, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), cell,
+			"text", 0,
+			NULL);
+
 	bcs = BACON_CD_SELECTION (widget);
+	cdrom_combo_box (bcs);
 
-#ifndef HAVE_GTK_ONLY
-	if (bcs->priv->is_entry)
-	{
-		bcs->priv->widget = gnome_file_entry_new (NULL,
-					_("Select the drive"));
-		g_object_set (G_OBJECT (bcs->priv->widget),
-				"use_filechooser", TRUE, NULL);
-		g_signal_connect (G_OBJECT (bcs->priv->widget), "changed",
-				G_CALLBACK (on_combo_entry_changed), bcs);
-
-		gtk_box_pack_start (GTK_BOX (widget),
-				bcs->priv->widget,
-				TRUE,       /* expand */
-				TRUE,       /* fill */
-				0);         /* padding */
-	} else
-#endif /* !HAVE_GTK_ONLY */
-	{
-		bcs->priv->widget = cdrom_option_menu (bcs);
-
-		g_signal_connect (bcs->priv->widget, "changed",
-				(GCallback)option_menu_device_changed, bcs);
-
-		gtk_box_pack_start (GTK_BOX (widget),
-				bcs->priv->widget,
-				TRUE,       /* expand */
-				TRUE,       /* fill */
-				0);         /* padding */
-	}
-
-	gtk_widget_show_all (bcs->priv->widget);
+	g_signal_connect (G_OBJECT (bcs), "changed",
+			G_CALLBACK (combo_device_changed), bcs);
 
 	return widget;
 }
 
+static void
+bacon_cd_selection_set_have_file_image (BaconCdSelection *bcs,
+		gboolean have_file_image)
+{
+	CDDrive *cdrom;
+
+	g_return_if_fail (bcs != NULL);
+	g_return_if_fail (BACON_IS_CD_SELECTION (bcs));
+
+	if (bcs->priv->have_file_image == have_file_image) {
+		return;
+	}
+
+	if (have_file_image == FALSE)
+	{
+		GList *item;
+		int index;
+
+		index = g_list_length (bcs->priv->cdroms);
+		gtk_combo_box_remove_text (GTK_COMBO_BOX (bcs), index);
+
+		item = g_list_last (bcs->priv->cdroms);
+		cdrom = (CDDrive *)item->data;
+		cd_drive_free (cdrom);
+		bcs->priv->cdroms = g_list_delete_link
+			(bcs->priv->cdroms, item);
+	} else {
+		cdrom = cd_drive_get_file_image ();
+		gtk_combo_box_append_text (GTK_COMBO_BOX (bcs),
+				cdrom->display_name);
+		bcs->priv->cdroms = g_list_append (bcs->priv->cdroms, cdrom);
+	}
+
+	bcs->priv->have_file_image = have_file_image;
+}
+
 /* Properties */
 static void
 bacon_cd_selection_set_property (GObject *object, guint property_id,
@@ -346,6 +275,10 @@ bacon_cd_selection_set_property (GObject *object, guint property_id,
 	case PROP_DEVICE:
 		bacon_cd_selection_set_device (bcs, g_value_get_string (value));
 		break;
+	case PROP_FILE_IMAGE:
+		bacon_cd_selection_set_have_file_image (bcs,
+				g_value_get_boolean (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 	}
@@ -366,6 +299,9 @@ bacon_cd_selection_get_property (GObject *object, guint property_id,
 	case PROP_DEVICE:
 		g_value_set_string (value, bacon_cd_selection_get_device (bcs));
 		break;
+	case PROP_FILE_IMAGE:
+		g_value_set_boolean (value, bcs->priv->have_file_image);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 	}
@@ -377,6 +313,9 @@ bacon_cd_selection_get_default_device (BaconCdSelection *bcs)
 	GList *l;
 	CDDrive *drive;
 
+	g_return_val_if_fail (bcs != NULL, "/dev/cdrom");
+	g_return_val_if_fail (BACON_IS_CD_SELECTION (bcs), "/dev/cdrom");
+
 	l = bcs->priv->cdroms;
 	if (bcs->priv->cdroms == NULL)
 		return "/dev/cdrom";
@@ -394,54 +333,39 @@ bacon_cd_selection_set_device (BaconCdSelection *bcs, const char *device)
 	gboolean found;
 	int i;
 
+	found = FALSE;
+	i = -1;
+
 	g_return_if_fail (bcs != NULL);
 	g_return_if_fail (BACON_IS_CD_SELECTION (bcs));
 
-#ifndef HAVE_GTK_ONLY
-	if (bcs->priv->is_entry != FALSE)
+	for (l = bcs->priv->cdroms; l != NULL && found == FALSE;
+			l = l->next)
 	{
-		GtkWidget *entry;
+		i++;
 
-		entry = gnome_file_entry_gtk_entry
-			(GNOME_FILE_ENTRY (bcs->priv->widget));
-		gtk_entry_set_text (GTK_ENTRY (entry), device);
-	} else
-#endif /* !HAVE_GTK_ONLY */
+		drive = l->data;
+
+		if (strcmp (drive->device, device) == 0)
+			found = TRUE;
+	}
+
+	if (found)
 	{
-		i = -1;
-		found = FALSE;
+		gtk_combo_box_set_active (GTK_COMBO_BOX (bcs), i);
+	} else {
+		/* If the device doesn't exist, set it back to
+		 * the default */
+		gtk_combo_box_set_active (GTK_COMBO_BOX (bcs), 0);
 
-		for (l = bcs->priv->cdroms; l != NULL && found == FALSE;
-				l = l->next)
-		{
-			i++;
+		drive = get_drive (bcs, 0);
 
-			drive = l->data;
+		if (drive == NULL)
+			return;
 
-			if (strcmp (drive->device, device) == 0)
-				found = TRUE;
-		}
-
-		if (found)
-		{
-			gtk_option_menu_set_history (GTK_OPTION_MENU
-					(bcs->priv->widget), i);
-		} else {
-			/* If the device doesn't exist, set it back to
-			 * the default */
-			gtk_option_menu_set_history (GTK_OPTION_MENU
-					(bcs->priv->widget), 0);
-
-			drive = get_drive (bcs, 0);
-
-			if (drive == NULL)
-				return;
-
-			g_signal_emit (G_OBJECT (bcs),
-					bcs_table_signals [DEVICE_CHANGED],
-					0, drive->device);
-		}
-			
+		g_signal_emit (G_OBJECT (bcs),
+				bcs_table_signals [DEVICE_CHANGED],
+				0, drive->device);
 	}
 }
 
@@ -454,25 +378,10 @@ bacon_cd_selection_get_device (BaconCdSelection *bcs)
 	g_return_val_if_fail (bcs != NULL, NULL);
 	g_return_val_if_fail (BACON_IS_CD_SELECTION (bcs), NULL);
 
-#ifndef HAVE_GTK_ONLY
-	if (bcs->priv->is_entry != FALSE)
-	{
-		GtkWidget *entry;
+	i = gtk_combo_box_get_active (GTK_COMBO_BOX (bcs));
+	drive = get_drive (bcs, i);
 
-		entry = gnome_file_entry_gtk_entry
-			(GNOME_FILE_ENTRY (bcs->priv->widget));
-		return gtk_entry_get_text (GTK_ENTRY (entry));
-	} else
-#endif /* !HAVE_GTK_ONLY */
-	{
-		i = gtk_option_menu_get_history (GTK_OPTION_MENU
-				(bcs->priv->widget));
-		drive = get_drive (bcs, i);
-
-		return drive ? drive->device : NULL;
-	}
-
-	return NULL;
+	return drive ? drive->device : NULL;
 }
 
 const CDDrive *
@@ -484,17 +393,9 @@ bacon_cd_selection_get_cdrom (BaconCdSelection *bcs)
 	g_return_val_if_fail (bcs != NULL, NULL);
 	g_return_val_if_fail (BACON_IS_CD_SELECTION (bcs), NULL);
 
-	if (bcs->priv->is_entry != FALSE)
-	{
-		return NULL;
-	} else {
-		i = gtk_option_menu_get_history (GTK_OPTION_MENU
-				(bcs->priv->widget));
-		drive = get_drive (bcs, i);
+	i = gtk_combo_box_get_active (GTK_COMBO_BOX (bcs));
+	drive = get_drive (bcs, i);
 
-		return drive;
-	}
-
-	return NULL;
+	return drive;
 }
 
